@@ -1,6 +1,6 @@
 /* common.c - common functions used by daisy-player and eBook-speaker.
  *
- * Copyright (C)2017 J. Lemmens
+ * Copyright (C)2018 J. Lemmens
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -19,29 +19,102 @@
 
 #include "daisy.h"
 
-void set_volume (misc_t *misc)
+char *convert_URL_name (misc_t *misc, char *file)
 {
-   snd_mixer_t *handle;
-   snd_mixer_selem_id_t *sid;
-   snd_mixer_elem_t *elem;
+   int i, j;
 
-   if (snd_mixer_open (&handle, 0) != 0)
-      failure (misc, "snd_mixer_open", errno);
-   snd_mixer_attach (handle, misc->sound_dev);
-   snd_mixer_selem_register (handle, NULL, NULL);
-   snd_mixer_load (handle);
-   snd_mixer_selem_id_alloca (&sid);
-   snd_mixer_selem_id_set_index (sid, 0);
-   snd_mixer_selem_id_set_name (sid, "Master");
-   if ((elem = snd_mixer_find_selem (handle, sid)) == NULL)
-      failure (misc, "No ALSA device found", errno);
-   snd_mixer_selem_set_playback_volume_all (elem, misc->volume);
-   snd_mixer_close (handle);
-} // set_volume
+   *misc->str = j = 0;
+   for (i = 0; i < (int) strlen (file) - 2; i++)
+   {
+      if (file[i] == '%' && isxdigit (file[i + 1]) && isxdigit (file[i + 2]))
+      {
+         char hex[10];
+
+         hex[0] = '0';
+         hex[1] = 'x';
+         hex[2] = file[i + 1];
+         hex[3] = file[i + 2];
+         hex[4] = 0;
+         sscanf (hex, "%x", (unsigned int *) &misc->str[j++]);
+         i += 2;
+      }
+      else
+         misc->str[j++] = file[i];
+   } // for
+   misc->str[j++] = file[i++];
+   misc->str[j++] = file[i];
+   misc->str[j] = 0;
+   return misc->str;
+} // convert_URL_name
+
+void failure (misc_t *misc, char *str, int e)
+{
+   endwin ();
+   beep ();
+   printf ("\n\n%s: %s\n", str, strerror (e));
+   fflush (stdout);
+   remove_tmp_dir (misc);
+   kill (misc->main_pid, SIGTERM);
+} // failure
+
+void player_ended ()
+{
+   pid_t kidpid;
+   int status;
+
+   while ((kidpid = waitpid(-1, &status, WNOHANG)) > 0)
+   {
+   } // while
+} // player_ended
+
+void get_path_name (char *dir, char *search_str, char *found)
+{
+   int total, n;
+   struct dirent **namelist;
+
+   total = scandir ((const char *) dir, &namelist, NULL, alphasort);
+   if (total == -1)
+   {
+      printf ("scandir: %s: %s\n", dir, strerror (errno));
+      exit (-1);
+   } // if
+   for (n = 0; n < total; n++)
+   {
+      char *path;
+
+      path = malloc (strlen (dir) + strlen (namelist[n]->d_name) + 2);
+      if (dir[strlen (dir) - 1] == '/')
+         sprintf (path, "%s%s", dir, namelist[n]->d_name);
+      else
+         sprintf (path, "%s/%s", dir, namelist[n]->d_name);
+      if (strcasecmp (namelist[n]->d_name, basename (search_str)) == 0)  
+      {
+         strcpy (found, path);
+         return;
+      } // if
+      free (path);
+      if (strcmp (namelist[n]->d_name, ".") == 0 ||
+          strcmp (namelist[n]->d_name, "..") == 0)
+         continue;
+      if (namelist[n]->d_type == DT_DIR)
+      {
+         char *new_dir;
+
+         new_dir = malloc (strlen (dir) + strlen (namelist[n]->d_name) + 2);
+         if (dir[strlen (dir) - 1] != '/')
+            sprintf (new_dir, "%s/%s", dir, namelist[n]->d_name);
+         else
+            sprintf (new_dir, "%s%s", dir, namelist[n]->d_name);
+         get_path_name (new_dir, search_str, found);
+         free (new_dir);
+      } // if
+   } // for
+   free (namelist);
+} // get_path_name
 
 char *get_dir_content (misc_t *misc, char *dir_name, char *search_str)
 {
-   char *found;
+   char *found;                                                       
    DIR *dir;
    struct dirent *entry;
 
@@ -83,183 +156,9 @@ char *get_dir_content (misc_t *misc, char *dir_name, char *search_str)
    return "";
 } // get_dir_content
 
-char *get_real_name (misc_t *misc, char *dir, char *name)
-{
-   char *found;
-   DIR *DIR;
-   struct dirent *entry;
-
-   if (! (DIR = opendir (dir)))
-      failure (misc, name, errno);
-   if (! (entry = readdir (DIR)))
-      failure (misc, "readdir ()", errno);
-   found = strdup ("");
-   do
-   {
-      if (strcasecmp (entry->d_name, name) == 0)
-      {
-         found = malloc (strlen (dir) + strlen (entry->d_name) + 5);
-         sprintf (found, "%s/%s\n", dir, entry->d_name);
-         found[strlen (found) - 1] = 0;
-         closedir (DIR);
-         return found;
-      } // if
-   } while ((entry = readdir (DIR)));
-   endwin ();
-   beep ();
-   printf ("   \"%s\": %s\n\n", name, strerror (ENOENT));
-   printf ("%s\n", gettext ("eBook-speaker cannot handle this file."));
-   fflush (stdout);
-   remove_tmp_dir (misc);
-   _exit (-1);
-} // get_real_name
-
-char *real_name (misc_t *misc, char *file)
-{
-// open file case insensitively
-   char *dir, *path;
-   int i, j;
-
-   *misc->str = j = 0;
-   for (i = 0; i < (int) strlen (file) - 2; i++)
-   {
-      if (file[i] == '%' && isxdigit (file[i + 1]) && isxdigit (file[i + 2]))
-      {
-         char hex[10];
-
-         hex[0] = '0';
-         hex[1] = 'x';
-         hex[2] = file[i + 1];
-         hex[3] = file[i + 2];
-         hex[4] = 0;
-         sscanf (hex, "%x", (unsigned int *) &misc->str[j++]);
-         i += 2;
-      }
-      else
-         misc->str[j++] = file[i];
-   } // for
-   misc->str[j++] = file[i++];
-   misc->str[j++] = file[i];
-   misc->str[j] = 0;
-   file = misc->str;
-   dir  = strdup (file);
-   dir  = dirname (dir);
-   path = get_real_name (misc, dir, basename (file));
-   return path;
-} // real_name
-
-void failure (misc_t *misc, char *str, int e)
-{
-   endwin ();
-   beep ();
-   fprintf (stderr, "\n%s: %s\n", str, strerror (e));
-   fflush (stdout);
-   remove_tmp_dir (misc);
-   _exit (-1);
-} // failure
-
-void playfile (misc_t *misc, char *in_file, char *in_type,
-               char *out_file, char *out_type, char *tempo)
-{
-   sox_format_t *sox_in, *sox_out;
-   sox_effects_chain_t *chain;
-   sox_effect_t *e;
-   char *args[MAX_STR];
-
-   sox_globals.verbosity = 0;
-   sox_globals.stdout_in_use_by = NULL;
-   sox_init ();
-   if ((sox_in = sox_open_read (in_file, NULL, NULL, in_type)) == NULL)
-   {
-      int e;
-
-      e = errno;
-      beep ();
-      endwin ();
-      printf ("\n%s: %s\n", in_file, strerror (e));
-      fflush (stdout);
-      remove_tmp_dir (misc);
-      kill (0, SIGTERM);
-   } // if
-   if ((sox_out = sox_open_write (out_file, &sox_in->signal,
-                                  NULL, out_type, NULL, NULL)) == NULL)
-   {
-      {
-         beep ();
-         endwin ();
-         printf ("\n%s: %s\n", out_file, strerror (EBUSY));
-         kill (0, SIGTERM);
-      } // if
-   } // if
-   if (strcmp (in_type, "cdda") == 0)
-   {
-      sox_in->encoding.encoding = SOX_ENCODING_SIGN2;
-      sox_in->encoding.bits_per_sample = 16;
-      sox_in->encoding.reverse_bytes = sox_option_no;
-   } // if
-
-   chain = sox_create_effects_chain (&sox_in->encoding, &sox_out->encoding);
-
-   e = sox_create_effect (sox_find_effect ("input"));
-   args[0] = (char *) sox_in, sox_effect_options (e, 1, args);
-   sox_add_effect (chain, e, &sox_in->signal, &sox_in->signal);
-
-/* Don't use the sox trim effect. It works nice, but is far too slow
-   char str2[MAX_STR];
-   snprintf (str,  MAX_STR - 1, "%f", misc->clip_begin);
-   snprintf (str2, MAX_STR - 1, "%f", misc->clip_end - misc->clip_begin);
-   e = sox_create_effect (sox_find_effect ("trim"));
-   args[0] = str;
-   args[1] = str2;
-   sox_effect_options (e, 2, args);
-   sox_add_effect (chain, e, &sox_in->signal, &sox_in->signal);
-*/
-
-   e = sox_create_effect (sox_find_effect ("tempo"));
-#ifdef DAISY_PLAYER
-   if (misc->cd_type == CDIO_DISC_MODE_CD_DA)
-      args[0] = "-m";
-   else
-#endif
-      args[0] = "-s";
-   args[1] = tempo;
-   sox_effect_options (e, 2, args);
-   sox_add_effect (chain, e, &sox_in->signal, &sox_in->signal);
-
-   e = sox_create_effect (sox_find_effect ("rate"));
-   snprintf (misc->str, MAX_STR - 1, "%lf", sox_out->signal.rate);
-   args[0] = misc->str, sox_effect_options (e, 1, args);
-   sox_add_effect (chain, e, &sox_in->signal, &sox_in->signal);
-
-   snprintf (misc->str, MAX_STR - 1, "%i", sox_out->signal.channels);
-   e = sox_create_effect (sox_find_effect ("channels"));
-   args[0] = misc->str, sox_effect_options (e, 1, args);
-   sox_add_effect (chain, e, &sox_in->signal, &sox_in->signal);
-
-   e = sox_create_effect (sox_find_effect ("output"));
-   args[0] = (char *) sox_out, sox_effect_options (e, 1, args);
-   sox_add_effect (chain, e, &sox_in->signal, &sox_out->signal);
-
-   sox_flow_effects (chain, NULL, NULL);
-   sox_delete_effects_chain (chain);
-   sox_close (sox_out);
-   sox_close (sox_in);
-   unlink (in_file);
-   sox_quit ();
-   unlink (misc->tmp_wav);
-} // playfile
-
-void player_ended ()
-{
-   wait (NULL);
-} // player_ended
-
 void find_index_names (misc_t *misc)
 {
-   *misc->ncc_html = 0;
-   strncpy (misc->ncc_html,
-            get_dir_content (misc, misc->daisy_mp, "ncc.html"),
-            MAX_STR - 1);
+   get_path_name (misc->daisy_mp, "ncc.html", misc->ncc_html);
    *misc->ncx_name = 0;
    strncpy (misc->ncx_name,
             get_dir_content (misc, misc->daisy_mp, ".ncx"),
@@ -282,15 +181,13 @@ int get_page_number_2 (misc_t *misc, my_attribute_t *my_attribute,
    htmlDocPtr doc;
 
    src = malloc (strlen (misc->daisy_mp) + strlen (attr) + 3);
-   strcpy (src, misc->daisy_mp);
-   strcat (src, "/");
-   strcat (src, attr);
    anchor = strdup ("");
-   if (strchr (src, '#'))
+   if (strchr (attr, '#'))
    {
-      anchor = strdup (strchr (src, '#') + 1);
-      *strchr (src, '#') = 0;
+      anchor = strdup (strchr (attr, '#') + 1);
+      *strchr (attr, '#') = 0;
    } // if
+   get_path_name (misc->daisy_mp, attr, src);
    doc = htmlParseFile (src, "UTF-8");
    if (! (page = xmlReaderWalker (doc)))
    {
@@ -366,7 +263,10 @@ int get_page_number_3 (misc_t *misc, my_attribute_t *my_attribute)
             anchor = strdup (strchr (my_attribute->src, '#') + 1);
             *strchr (my_attribute->src, '#') = 0;
          } // if
-         file = real_name (misc, my_attribute->src);
+         file = malloc (strlen (misc->daisy_mp) +
+                        strlen (my_attribute->src) + 5);
+         get_path_name (misc->daisy_mp,
+                        convert_URL_name (misc, my_attribute->src), file);
          doc = htmlParseFile (file, "UTF-8");
          if (! (page = xmlReaderWalker (doc)))
          {
@@ -413,21 +313,23 @@ int get_page_number_3 (misc_t *misc, my_attribute_t *my_attribute)
       {
          misc->current_page_number = atoi (misc->label);
          return 1;
-      } // if                                 
+      } // if
       if (! get_tag_or_label (misc, my_attribute, misc->reader))
          return 0;
 #endif
    } // while
-} // get_page_number_3
+} // get_page_number_3                                         
 
 void kill_player (misc_t *misc)
 {
-   kill (misc->player_pid, SIGKILL);
+   while (kill (misc->player_pid, SIGHUP) == 0);
 #ifdef EBOOK_SPEAKER
    unlink (misc->eBook_speaker_txt);
    unlink (misc->tmp_wav);
 #endif
 #ifdef DAISY_PLAYER
+   if (misc->cd_type == CDIO_DISC_MODE_CD_DA)
+      while (kill (misc->cdda_pid, SIGKILL) == 0);
    unlink (misc->tmp_wav);
 #endif
 } // kill_player
@@ -445,12 +347,164 @@ void skip_right (misc_t *misc, daisy_t *daisy)
    } // if
    misc->current = misc->displaying = misc->playing;
    wmove (misc->screenwin, daisy[misc->current].y, daisy[misc->current].x);
+   wrefresh (misc->screenwin);
    kill_player (misc);
 } // skip_right
 
+#ifdef DAISY_PLAYER
+int handle_ncc_html (misc_t *misc, my_attribute_t *my_attribute,
+                     daisy_t *daisy)
+#endif
+#ifdef EBOOK_SPEAKER
+int handle_ncc_html (misc_t *misc, my_attribute_t *my_attribute)
+#endif
+{
+// lookfor "ncc.html"
+   htmlDocPtr doc;
+   xmlTextReaderPtr ncc;
+
+   misc->daisy_mp = strdup (misc->ncc_html);
+   misc->daisy_mp = dirname (misc->daisy_mp);
+   strncpy (misc->daisy_version, "2.02", 4);
+   doc = htmlParseFile (misc->ncc_html, "UTF-8");
+   ncc = xmlReaderWalker (doc);
+   misc->total_items = 0;
+   while (1)
+   {
+      if (! get_tag_or_label (misc, my_attribute, ncc))
+         break;
+      if (strcasecmp (misc->tag, "h1") == 0 ||
+          strcasecmp (misc->tag, "h2") == 0 ||
+          strcasecmp (misc->tag, "h3") == 0 ||
+          strcasecmp (misc->tag, "h4") == 0 ||
+          strcasecmp (misc->tag, "h5") == 0 ||
+          strcasecmp (misc->tag, "h6") == 0)
+      {
+         misc->total_items++;
+      } // if
+   } // while
+   xmlTextReaderClose (ncc);
+   xmlFreeDoc (doc);
+   if (misc->total_items == 0)
+   {
+      endwin ();
+      printf ("%s\n",
+              gettext ("Please try to play this book with daisy-player"));
+      beep ();
+#ifdef DAISY_PLAYER
+      quit_daisy_player (misc, daisy);
+#endif
+#ifdef EBOOK_SPEAKER
+      quit_eBook_speaker (misc);
+#endif
+      _exit (0);
+   } // if
+   return misc->total_items;
+} // handle_ncc_html
+
+int namefilter (const struct dirent *namelist)
+{
+   int  r = 0;
+   char     my_pattern[] = "*.smil";
+
+   r = fnmatch (my_pattern, namelist->d_name, FNM_PERIOD);
+   return (r == 0) ? 1 : 0;
+} // namefilter
+
+int get_meta_attributes (xmlTextReaderPtr parse, xmlTextWriterPtr writer)
+{
+   while (1)
+   {
+      char *name, *content;
+
+      name = (char *) xmlTextReaderGetAttribute (parse,
+                            (const unsigned char *) "name");
+      if (! name)
+         return 0;
+      content = (char *) xmlTextReaderGetAttribute (parse,
+                     (const unsigned char *) "content");
+      if (strcasecmp (name, "title") == 0)
+      {
+         xmlTextWriterWriteString (writer, (const xmlChar *)content);
+         return 1;
+      } // if
+      if (xmlTextReaderRead (parse) == 0)
+         return 0;
+      if (xmlTextReaderRead (parse) == 0)
+         return 0;
+   } // while
+} // get_meta_attributes
+
+void create_ncc_html (misc_t *misc)
+{
+   xmlTextWriterPtr writer;
+   int total, n;
+   struct dirent **namelist;
+
+   sprintf (misc->ncc_html, "%s/ncc.html", misc->daisy_mp);
+   if (! (writer = xmlNewTextWriterFilename (misc->ncc_html, 0)))
+      failure (misc, misc->ncc_html, errno);
+   xmlTextWriterSetIndent (writer, 1);
+   xmlTextWriterSetIndentString (writer, BAD_CAST "   ");
+   xmlTextWriterStartDocument (writer, NULL, NULL, NULL);
+   xmlTextWriterStartElement (writer, BAD_CAST "html");
+   xmlTextWriterWriteString (writer, BAD_CAST "\n");
+   xmlTextWriterStartElement (writer, BAD_CAST "head");
+   xmlTextWriterWriteString (writer, BAD_CAST "\n   ");
+   xmlTextWriterEndElement (writer);
+   xmlTextWriterStartElement (writer, BAD_CAST "body");
+   xmlTextWriterWriteString (writer, (const xmlChar *)"\n");
+   total = scandir (get_current_dir_name (), &namelist, namefilter,
+                    alphasort);
+   for (n = 0; n < total; n++)
+   {
+      xmlTextWriterStartElement (writer, BAD_CAST "h1");
+      xmlTextWriterWriteString (writer, BAD_CAST "\n");
+      xmlTextWriterStartElement (writer, BAD_CAST "a");
+      xmlTextWriterWriteFormatAttribute (writer, BAD_CAST "href",
+                                         "%s", namelist[n]->d_name);
+
+// write label
+      {
+      xmlTextReaderPtr parse;
+      char *str, *tag;
+
+      str = malloc (strlen (misc->daisy_mp) + strlen (namelist[n]->d_name) + 5);
+      sprintf (str, "%s/%s", misc->daisy_mp, namelist[n]->d_name);
+      parse = xmlReaderForFile (str, "UTF-8", 0);
+      while (1)
+      {
+         if (xmlTextReaderRead (parse) == 0)
+            break;
+         tag = (char *) xmlTextReaderConstName (parse);
+         if (strcasecmp (tag, "/head") == 0)
+            break;
+         if (strcasecmp (tag, "body") == 0)
+            break;
+         if (strcasecmp (tag, "meta") == 0)
+         {
+            if (get_meta_attributes (parse, writer))
+               break;
+         } // if
+      } // while
+      xmlTextReaderClose (parse);
+      free (str);
+      } // write label
+
+      xmlTextWriterWriteString (writer, BAD_CAST "\n         ");
+      xmlTextWriterEndElement (writer);
+      xmlTextWriterEndElement (writer);
+   } // for
+   xmlTextWriterEndElement (writer);
+   xmlTextWriterEndElement (writer);
+   xmlTextWriterEndDocument (writer);
+   xmlFreeTextWriter (writer);
+   free (namelist);
+} // create_ncc_html
+
 #ifdef DAISY_PLAYER
 daisy_t *create_daisy_struct (misc_t *misc, my_attribute_t *my_attribute,
-                              daisy_t *daisy)
+                          daisy_t *daisy)
 #endif
 #ifdef EBOOK_SPEAKER
 daisy_t *create_daisy_struct (misc_t *misc, my_attribute_t *my_attribute)
@@ -467,32 +521,13 @@ daisy_t *create_daisy_struct (misc_t *misc, my_attribute_t *my_attribute)
 // lookfor "ncc.html"
    if (*misc->ncc_html)
    {
-      htmlDocPtr doc;
-      xmlTextReaderPtr ncc;
-
-      misc->daisy_mp = strdup (misc->ncc_html);
-      misc->daisy_mp = dirname (misc->daisy_mp);
-      strncpy (misc->daisy_version, "2.02", 4);
-      doc = htmlParseFile (misc->ncc_html, "UTF-8");
-      ncc = xmlReaderWalker (doc);
-      misc->total_items = 0;
-      while (1)
-      {
-         if (! get_tag_or_label (misc, my_attribute, ncc))
-            break;
-         if (strcasecmp (misc->tag, "h1") == 0 ||
-             strcasecmp (misc->tag, "h2") == 0 ||
-             strcasecmp (misc->tag, "h3") == 0 ||
-             strcasecmp (misc->tag, "h4") == 0 ||
-             strcasecmp (misc->tag, "h5") == 0 ||
-             strcasecmp (misc->tag, "h6") == 0)
-         {
-            misc->total_items++;
-         } // if
-      } // while
-      xmlTextReaderClose (ncc);
-      xmlFreeDoc (doc);
-      return (daisy_t *) malloc ((misc->total_items + 1) * sizeof (daisy_t));
+#ifdef DAISY_PLAYER
+      misc->total_items = handle_ncc_html (misc, my_attribute, daisy);
+#endif
+#ifdef EBOOK_SPEAKER
+      misc->total_items = handle_ncc_html (misc, my_attribute);
+#endif
+      return calloc (misc->total_items + 1, sizeof (daisy_t));
    } // if ncc.html
 
 // lookfor *.ncx
@@ -509,11 +544,14 @@ daisy_t *create_daisy_struct (misc_t *misc, my_attribute_t *my_attribute)
 
    if (*misc->ncc_html == 0 && *misc->ncx_name == 0 && *misc->opf_name == 0)
    {
-      beep ();
-      endwin ();
-      printf ("%s\n", gettext (
-        "This book has no audio. Play this book with eBook-speaker"));
-      _exit (0);
+      create_ncc_html (misc);
+#ifdef DAISY_PLAYER
+      misc->total_items = handle_ncc_html (misc, my_attribute, daisy);
+#endif
+#ifdef EBOOK_SPEAKER
+      misc->total_items = handle_ncc_html (misc, my_attribute);
+#endif
+      return (daisy_t *) calloc (misc->total_items + 1, sizeof (daisy_t));
    } // if
 
 // count items in opf
@@ -563,15 +601,7 @@ daisy_t *create_daisy_struct (misc_t *misc, my_attribute_t *my_attribute)
    if (misc->items_in_opf > misc->items_in_ncx)
       misc->total_items = misc->items_in_opf;
    switch (chdir (misc->daisy_mp));
-#ifdef EBOOK_SPEAKER
-   snprintf (misc->eBook_speaker_txt, MAX_STR,
-             "%s/eBook-speaker.txt", misc->daisy_mp);
-   snprintf (misc->tmp_wav, MAX_STR,
-             "%s/eBook-speaker.wav", misc->daisy_mp);
-#endif
-   if (misc->total_items == 0)
-      misc->total_items = 1;
-   return (daisy_t *) malloc (misc->total_items * sizeof (daisy_t));
+   return (daisy_t *) calloc (misc->total_items + 1, sizeof (daisy_t));
 } // create_daisy_struct
 
 void make_tmp_dir (misc_t *misc)
@@ -579,12 +609,12 @@ void make_tmp_dir (misc_t *misc)
    misc->tmp_dir = strdup ("/tmp/" PACKAGE ".XXXXXX");
    if (! mkdtemp (misc->tmp_dir))
    {
-      int e;
+      int e;                                        
 
       e = errno;
       beep ();
       failure (misc, misc->tmp_dir, e);
-   } // if      
+   } // if
 #ifdef EBOOK_SPEAKER
    switch (chdir (misc->tmp_dir))
    {
@@ -598,29 +628,32 @@ void make_tmp_dir (misc_t *misc)
 
 void remove_tmp_dir (misc_t *misc)
 {
-   if (strncmp (misc->tmp_dir, "/tmp/", 5) == 0)
-   {
+   if (! misc->tmp_dir)
+      return;
+   if (strncmp (misc->tmp_dir, "/tmp/", 5) != 0)
+      return;
+
 // Be sure not to remove wrong files
 #ifdef DAISY_PLAYER
-      if (misc->cd_type == CDIO_DISC_MODE_CD_DA)
-      {
-         rmdir (misc->tmp_dir);
-         return;
-      } // if
+   if (misc->cd_type == CDIO_DISC_MODE_CD_DA)
+   {
+      rmdir (misc->tmp_dir);
+      return;
+   } // if
 #endif
 
-      snprintf (misc->cmd, MAX_CMD - 1, "rm -rf %s", misc->tmp_dir);
-      if (system (misc->cmd) != 0)
-      {
-         int e;
+   snprintf (misc->cmd, MAX_CMD - 1, "rm -rf %s", misc->tmp_dir);
+   if (system (misc->cmd) != 0)
+   {
+      int e;
+      char *str;
 
-         e = errno;
-         snprintf (misc->cmd, MAX_CMD + strlen (strerror (e)),
-                   "%s: %s\n", misc->cmd, strerror (e));
-         failure (misc, misc->str, e);
-      } // if
+      e = errno;
+      str = malloc (strlen (misc->cmd) + strlen (strerror (e)) + 10);
+      sprintf (str, "%s: %s\n", misc->cmd, strerror (e));
+      failure (misc, str, e);
    } // if
-} // remove_tmp_dir               
+} // remove_tmp_dir
 
 void clear_tmp_dir (misc_t *misc)
 {
@@ -781,9 +814,9 @@ void get_attributes (misc_t *misc, my_attribute_t *my_attribute,
    if (strcmp (attr, "(null)"))
       strncpy (my_attribute->smilref, attr, MAX_STR - 1);
    snprintf (attr, MAX_STR - 1, "%s", (char *)
-           xmlTextReaderGetAttribute (ptr, (const xmlChar *) "sound_dev"));
+           xmlTextReaderGetAttribute (ptr, (const xmlChar *) "pulseaudio_device"));
    if (strcmp (attr, "(null)"))
-      strncpy (misc->sound_dev, attr, MAX_STR - 1);
+      misc->pulseaudio_device = strdup (attr);
    snprintf (attr, MAX_STR - 1, "%s", (char *)
         xmlTextReaderGetAttribute (ptr, (const xmlChar *) "ocr_language"));
    if (strcmp (attr, "(null)"))
@@ -840,7 +873,7 @@ int get_tag_or_label (misc_t *misc, my_attribute_t *my_attribute,
    *my_attribute->clip_begin = *my_attribute->clip_end = 0;
 #endif
    *my_attribute->href = *my_attribute->media_type =
-   *my_attribute->playorder = * my_attribute->smilref =
+   *my_attribute->playorder = *my_attribute->smilref =
    *my_attribute->toc = *my_attribute->value = 0;
    my_attribute->id = strdup ("");
    my_attribute->idref = strdup ("");
@@ -941,7 +974,7 @@ void go_to_page_number (misc_t *misc, my_attribute_t *my_attribute,
 {
    char pn[15];
 
-   while (kill (misc->player_pid, SIGKILL) == 0);
+   kill_player (misc);
 #ifdef DAISY_PLAYER
    if (misc->cd_type != CDIO_DISC_MODE_CD_DA)
       misc->player_pid = -2;
@@ -1021,34 +1054,126 @@ void go_to_page_number (misc_t *misc, my_attribute_t *my_attribute,
    } // while
 } // go_to_page_number
 
-void select_next_output_device (misc_t *misc, my_attribute_t *my_attribute,
-                                daisy_t *daisy)
+void select_next_output_device (misc_t *misc, daisy_t *daisy)
 {
-   FILE *r;
-   int n, y, playing;
+   FILE *p;
+   int n, y, found;
    size_t bytes;
-   char *list[10], *trash;
+   char *card;
+   struct group *grp;
 
-   playing= misc->playing;
-   if (playing > -1)
-      pause_resume (misc, my_attribute, daisy);
    wclear (misc->screenwin);
    wprintw (misc->screenwin, "\n%s\n\n", gettext ("Select a soundcard:"));
-   if (! (r = fopen ("/proc/asound/cards", "r")))
-      failure (misc, gettext ("Cannot read /proc/asound/cards"), errno);
+   grp = getgrnam ("audio");
+   found = 0;
+   for (n = 0; grp->gr_mem[n]; n++)
+   {
+      if (strcmp (grp->gr_mem[n], cuserid (NULL)) == 0)
+      {
+         found = 1;
+         break;
+      } // if
+   } // for
+   if (found == 0)
+   {
+      beep ();
+      endwin ();
+      printf ("You need to be a member of the group \"audio\".\n");
+      printf ("Please give the following command:\n");
+      printf ("\n   sudo gpasswd -a <login-name> audio\n\n");
+      printf ("Logout and login again and you can use daisy-player.\n");
+      _exit (0);
+   } // if
+
+   grp = getgrnam ("pulse");
+   found = 0;
+   for (n = 0; grp->gr_mem[n]; n++)
+   {
+      if (strcmp (grp->gr_mem[n], cuserid (NULL)) == 0)
+      {
+         found = 1;
+         break;
+      } // if
+   } // for
+   if (found == 0)
+   {
+      beep ();
+      endwin ();
+      printf ("You need to be a member of the group \"pulse\".\n");
+      printf ("Please give the following command:\n");
+      printf ("\n   sudo gpasswd -a <login-name> pulse\n\n");
+      printf ("Logout and login again and you can use daisy-player.\n");
+      _exit (0);
+   } //; if
+
+   system ("pulseaudio --start");
+   if ((p = popen ("/usr/bin/pacmd list-cards", "r")) == NULL)
+   {
+      beep ();
+      endwin ();
+      printf ("%s\n",
+       "Be sure the package pulseaudio-utils is installed onto your system.");
+      _exit (-1);
+   } // if
    for (n = 0; n < 10; n++)
    {
-      list[n] = (char *) malloc (1000);
-      bytes = getline (&list[n], &bytes, r);
+      int m;
+
+      misc->pulseaudio_device = malloc (10);
+      sprintf (misc->pulseaudio_device, "%d", n);
+      do
+      {
+         card = NULL;
+         bytes = 0;
+         bytes = getline (&card, &bytes, p);
+         if ((int) bytes == -1)
+         {
+            card = NULL;
+            break;
+         } // if
+      } while (strcasestr (card, "index:") == NULL);
+
+      while (1)
+      {
+         card = NULL;
+         bytes = 0;
+         bytes = getline (&card, &bytes, p);
+         if ((int) bytes == -1)
+         {
+            card = NULL;
+            break;
+         } // if
+         if (strcasestr (card, "device.vendor.name"))
+         {
+            for (m = 0; m < (int) strlen (card); m++)
+               if (card[m] == '=')
+                  break;
+            card += m + 3;
+            if (strlen (card) > 2)
+               card[strlen (card) - 2] = 0;
+            wprintw (misc->screenwin, "   %d %s ", n, card);
+            wrefresh (misc->screenwin);
+            continue;
+         } // if
+
+         if (strcasestr (card, "device.product.name"))
+         {
+            for (m = 0; m < (int) strlen (card); m++)
+               if (card[m] == '=')
+                  break;
+            card += m + 3;
+            if (strlen (card) > 2)
+               card[strlen (card) - 2] = 0;
+            wprintw (misc->screenwin, "%s\n", card);
+            wrefresh (misc->screenwin);
+            break;
+         } // if
+      } // while
       if ((int) bytes == -1)
          break;
-      trash = (char *) malloc (1000);
-      bytes = getline (&trash, &bytes, r);
-      free (trash);
-      wprintw (misc->screenwin, "   %s", list[n]);
-      free (list[n]);
    } // for
-   fclose (r);
+   wrefresh (misc->screenwin);
+   fclose (p);
    y = 3;
    nodelay (misc->screenwin, FALSE);
    for (;;)
@@ -1057,11 +1182,10 @@ void select_next_output_device (misc_t *misc, my_attribute_t *my_attribute,
       switch (wgetch (misc->screenwin))
       {
       case 13: //
-         snprintf (misc->sound_dev, MAX_STR - 1, "hw:%i", y - 3);
+         misc->pulseaudio_device = malloc (10);
+         snprintf (misc->pulseaudio_device, 3, "%d", y - 3);
          view_screen (misc, daisy);
          nodelay (misc->screenwin, TRUE);
-         if (playing > -1)
-            pause_resume (misc, my_attribute, daisy);
          return;
       case KEY_DOWN:
          if (++y == n + 3)
@@ -1072,11 +1196,8 @@ void select_next_output_device (misc_t *misc, my_attribute_t *my_attribute,
             y = n + 2;
          break;
       default:
-         view_screen (misc, daisy);
-         nodelay (misc->screenwin, TRUE);
-         if (playing > -1)
-            pause_resume (misc, my_attribute, daisy);
-         return;
+         beep ();
+         break;
       } // switch
    } // for
 } // select_next_output_device
